@@ -12,4 +12,4 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from keystone.contrib.federation.core import * # flake8: noqa
+from keystone.contrib.federation.core import *  # noqa
