@@ -12,20 +12,21 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import json
+import time
 import uuid
 
+from keystoneclient.common import cms
+from oslo_config import cfg
 import six
-from six.moves import http_client
+from testtools import matchers
 
 from keystone.common import extension as keystone_extension
-import keystone.conf
-from keystone.tests import unit
-from keystone.tests.unit import default_fixtures
 from keystone.tests.unit import ksfixtures
 from keystone.tests.unit import rest
-from keystone.tests.unit.schema import v2
 
-CONF = keystone.conf.CONF
+
+CONF = cfg.CONF
 
 
 class CoreApiTests(object):
@@ -62,20 +63,20 @@ class CoreApiTests(object):
         self.assertIsNotNone(user.get('id'))
         self.assertIsNotNone(user.get('name'))
 
-    def assertValidRole(self, role):
-        self.assertIsNotNone(role.get('id'))
-        self.assertIsNotNone(role.get('name'))
+    def assertValidRole(self, tenant):
+        self.assertIsNotNone(tenant.get('id'))
+        self.assertIsNotNone(tenant.get('name'))
 
     def test_public_not_found(self):
         r = self.public_request(
             path='/%s' % uuid.uuid4().hex,
-            expected_status=http_client.NOT_FOUND)
+            expected_status=404)
         self.assertValidErrorResponse(r)
 
     def test_admin_not_found(self):
         r = self.admin_request(
             path='/%s' % uuid.uuid4().hex,
-            expected_status=http_client.NOT_FOUND)
+            expected_status=404)
         self.assertValidErrorResponse(r)
 
     def test_public_multiple_choice(self):
@@ -104,13 +105,13 @@ class CoreApiTests(object):
         self.assertValidExtensionListResponse(
             r, keystone_extension.ADMIN_EXTENSIONS)
 
-    def test_admin_extensions_returns_not_found(self):
+    def test_admin_extensions_404(self):
         self.admin_request(path='/v2.0/extensions/invalid-extension',
-                           expected_status=http_client.NOT_FOUND)
+                           expected_status=404)
 
-    def test_public_osksadm_extension_returns_not_found(self):
+    def test_public_osksadm_extension_404(self):
         self.public_request(path='/v2.0/extensions/OS-KSADM',
-                            expected_status=http_client.NOT_FOUND)
+                            expected_status=404)
 
     def test_admin_osksadm_extension(self):
         r = self.admin_request(path='/v2.0/extensions/OS-KSADM')
@@ -130,7 +131,7 @@ class CoreApiTests(object):
                     'tenantId': self.tenant_bar['id'],
                 },
             },
-            expected_status=http_client.OK)
+            expected_status=200)
         self.assertValidAuthenticationResponse(r, require_service_catalog=True)
 
     def test_authenticate_unscoped(self):
@@ -145,7 +146,7 @@ class CoreApiTests(object):
                     },
                 },
             },
-            expected_status=http_client.OK)
+            expected_status=200)
         self.assertValidAuthenticationResponse(r)
 
     def test_get_tenants_for_token(self):
@@ -162,14 +163,14 @@ class CoreApiTests(object):
             token=token)
         self.assertValidAuthenticationResponse(r)
 
-    def test_invalid_token_returns_not_found(self):
+    def test_invalid_token_404(self):
         token = self.get_scoped_token()
         self.admin_request(
             path='/v2.0/tokens/%(token_id)s' % {
                 'token_id': 'invalid',
             },
             token=token,
-            expected_status=http_client.NOT_FOUND)
+            expected_status=404)
 
     def test_validate_token_service_role(self):
         self.md_foobar = self.assignment_api.add_role_to_user_and_project(
@@ -177,8 +178,7 @@ class CoreApiTests(object):
             self.tenant_service['id'],
             self.role_service['id'])
 
-        token = self.get_scoped_token(
-            tenant_id=default_fixtures.SERVICE_TENANT_ID)
+        token = self.get_scoped_token(tenant_id='service')
         r = self.admin_request(
             path='/v2.0/tokens/%s' % token,
             token=token)
@@ -190,8 +190,7 @@ class CoreApiTests(object):
             self.tenant_service['id'],
             self.role_service['id'])
 
-        token = self.get_scoped_token(
-            tenant_id=default_fixtures.SERVICE_TENANT_ID)
+        token = self.get_scoped_token(tenant_id='service')
         r = self.admin_request(
             path='/v2.0/tokens/%s' % token,
             token=token)
@@ -205,7 +204,7 @@ class CoreApiTests(object):
         r = self.admin_request(
             path='/v2.0/tokens/%s' % token,
             token=token,
-            expected_status=http_client.UNAUTHORIZED)
+            expected_status=401)
 
     def test_validate_token_belongs_to(self):
         token = self.get_scoped_token()
@@ -234,7 +233,7 @@ class CoreApiTests(object):
                 'token_id': token,
             },
             token=token,
-            expected_status=http_client.OK)
+            expected_status=200)
 
     def test_endpoints(self):
         token = self.get_scoped_token()
@@ -273,14 +272,6 @@ class CoreApiTests(object):
             token=token)
         self.assertValidRoleListResponse(r)
 
-    def test_get_user_roles_without_tenant(self):
-        token = self.get_scoped_token()
-        self.admin_request(
-            path='/v2.0/users/%(user_id)s/roles' % {
-                'user_id': self.user_foo['id'],
-            },
-            token=token, expected_status=http_client.NOT_IMPLEMENTED)
-
     def test_get_user(self):
         token = self.get_scoped_token()
         r = self.admin_request(
@@ -315,7 +306,7 @@ class CoreApiTests(object):
                 },
             },
             token=token,
-            expected_status=http_client.BAD_REQUEST)
+            expected_status=400)
         self.assertValidErrorResponse(r)
 
         r = self.admin_request(
@@ -330,7 +321,7 @@ class CoreApiTests(object):
                 },
             },
             token=token,
-            expected_status=http_client.BAD_REQUEST)
+            expected_status=400)
         self.assertValidErrorResponse(r)
 
         # Test UPDATE request
@@ -347,7 +338,7 @@ class CoreApiTests(object):
                 },
             },
             token=token,
-            expected_status=http_client.BAD_REQUEST)
+            expected_status=400)
         self.assertValidErrorResponse(r)
 
         r = self.admin_request(
@@ -360,7 +351,7 @@ class CoreApiTests(object):
                 },
             },
             token=token,
-            expected_status=http_client.BAD_REQUEST)
+            expected_status=400)
         self.assertValidErrorResponse(r)
 
     def test_create_update_user_valid_enabled_type(self):
@@ -378,12 +369,11 @@ class CoreApiTests(object):
                                },
                            },
                            token=token,
-                           expected_status=http_client.OK)
+                           expected_status=200)
 
     def test_error_response(self):
-        """Trigger assertValidErrorResponse by convention."""
-        self.public_request(path='/v2.0/tenants',
-                            expected_status=http_client.UNAUTHORIZED)
+        """This triggers assertValidErrorResponse by convention."""
+        self.public_request(path='/v2.0/tenants', expected_status=401)
 
     def test_invalid_parameter_error_response(self):
         token = self.get_scoped_token()
@@ -397,13 +387,13 @@ class CoreApiTests(object):
                                  path='/v2.0/OS-KSADM/services',
                                  body=bad_body,
                                  token=token,
-                                 expected_status=http_client.BAD_REQUEST)
+                                 expected_status=400)
         self.assertValidErrorResponse(res)
         res = self.admin_request(method='POST',
                                  path='/v2.0/users',
                                  body=bad_body,
                                  token=token,
-                                 expected_status=http_client.BAD_REQUEST)
+                                 expected_status=400)
         self.assertValidErrorResponse(res)
 
     def _get_user_id(self, r):
@@ -443,7 +433,7 @@ class CoreApiTests(object):
         raise NotImplementedError()
 
     def assertNoRoles(self, r):
-        """Helper method to assert No Roles.
+        """Helper method to assert No Roles
 
         This needs to be overridden by child classes
         based on their content type.
@@ -467,7 +457,7 @@ class CoreApiTests(object):
                 },
             },
             token=token,
-            expected_status=http_client.OK)
+            expected_status=200)
 
         user_id = self._get_user_id(r.result)
 
@@ -478,7 +468,7 @@ class CoreApiTests(object):
                 'user_id': user_id
             },
             token=token,
-            expected_status=http_client.OK)
+            expected_status=200)
         self.assertEqual(CONF.member_role_name, self._get_role_name(r.result))
 
         # Create a new tenant
@@ -493,7 +483,7 @@ class CoreApiTests(object):
                 },
             },
             token=token,
-            expected_status=http_client.OK)
+            expected_status=200)
 
         project_id = self._get_project_id(r.result)
 
@@ -509,7 +499,7 @@ class CoreApiTests(object):
                 },
             },
             token=token,
-            expected_status=http_client.OK)
+            expected_status=200)
 
         # 'member_role' should be in new_tenant
         r = self.admin_request(
@@ -518,7 +508,7 @@ class CoreApiTests(object):
                 'user_id': user_id
             },
             token=token,
-            expected_status=http_client.OK)
+            expected_status=200)
         self.assertEqual('_member_', self._get_role_name(r.result))
 
         # 'member_role' should not be in tenant_bar any more
@@ -528,7 +518,7 @@ class CoreApiTests(object):
                 'user_id': user_id
             },
             token=token,
-            expected_status=http_client.OK)
+            expected_status=200)
         self.assertNoRoles(r.result)
 
     def test_update_user_with_invalid_tenant(self):
@@ -547,7 +537,7 @@ class CoreApiTests(object):
                 },
             },
             token=token,
-            expected_status=http_client.OK)
+            expected_status=200)
         user_id = self._get_user_id(r.result)
 
         # Update user with an invalid tenant
@@ -562,7 +552,7 @@ class CoreApiTests(object):
                 },
             },
             token=token,
-            expected_status=http_client.NOT_FOUND)
+            expected_status=404)
 
     def test_update_user_with_invalid_tenant_no_prev_tenant(self):
         token = self.get_scoped_token()
@@ -579,7 +569,7 @@ class CoreApiTests(object):
                 },
             },
             token=token,
-            expected_status=http_client.OK)
+            expected_status=200)
         user_id = self._get_user_id(r.result)
 
         # Update user with an invalid tenant
@@ -594,7 +584,7 @@ class CoreApiTests(object):
                 },
             },
             token=token,
-            expected_status=http_client.NOT_FOUND)
+            expected_status=404)
 
     def test_update_user_with_old_tenant(self):
         token = self.get_scoped_token()
@@ -612,7 +602,7 @@ class CoreApiTests(object):
                 },
             },
             token=token,
-            expected_status=http_client.OK)
+            expected_status=200)
 
         user_id = self._get_user_id(r.result)
 
@@ -623,7 +613,7 @@ class CoreApiTests(object):
                 'user_id': user_id
             },
             token=token,
-            expected_status=http_client.OK)
+            expected_status=200)
         self.assertEqual(CONF.member_role_name, self._get_role_name(r.result))
 
         # Update user's tenant with old tenant id
@@ -638,7 +628,7 @@ class CoreApiTests(object):
                 },
             },
             token=token,
-            expected_status=http_client.OK)
+            expected_status=200)
 
         # 'member_role' should still be in tenant_bar
         r = self.admin_request(
@@ -647,7 +637,7 @@ class CoreApiTests(object):
                 'user_id': user_id
             },
             token=token,
-            expected_status=http_client.OK)
+            expected_status=200)
         self.assertEqual('_member_', self._get_role_name(r.result))
 
     def test_authenticating_a_user_with_no_password(self):
@@ -679,13 +669,13 @@ class CoreApiTests(object):
                     },
                 },
             },
-            expected_status=http_client.UNAUTHORIZED)
+            expected_status=401)
         self.assertValidErrorResponse(r)
 
     def test_www_authenticate_header(self):
         r = self.public_request(
             path='/v2.0/tenants',
-            expected_status=http_client.UNAUTHORIZED)
+            expected_status=401)
         self.assertEqual('Keystone uri="http://localhost"',
                          r.headers.get('WWW-Authenticate'))
 
@@ -694,13 +684,13 @@ class CoreApiTests(object):
         self.config_fixture.config(public_endpoint=test_url)
         r = self.public_request(
             path='/v2.0/tenants',
-            expected_status=http_client.UNAUTHORIZED)
+            expected_status=401)
         self.assertEqual('Keystone uri="%s"' % test_url,
                          r.headers.get('WWW-Authenticate'))
 
 
 class LegacyV2UsernameTests(object):
-    """Test to show the broken username behavior in V2.
+    """Tests to show the broken username behavior in V2.
 
     The V2 API is documented to use `username` instead of `name`.  The
     API forced used to use name and left the username to fall into the
@@ -711,7 +701,7 @@ class LegacyV2UsernameTests(object):
     """
 
     def create_user(self, **user_attrs):
-        """Create a users and returns the response object.
+        """Creates a users and returns the response object.
 
         :param user_attrs: attributes added to the request body (optional)
         """
@@ -729,7 +719,7 @@ class LegacyV2UsernameTests(object):
             path='/v2.0/users',
             token=token,
             body=body,
-            expected_status=http_client.OK)
+            expected_status=200)
 
     def test_create_with_extra_username(self):
         """The response for creating a user will contain the extra fields."""
@@ -780,7 +770,7 @@ class LegacyV2UsernameTests(object):
                     'enabled': enabled,
                 },
             },
-            expected_status=http_client.OK)
+            expected_status=200)
 
         self.assertValidUserResponse(r)
 
@@ -810,7 +800,7 @@ class LegacyV2UsernameTests(object):
                     'enabled': enabled,
                 },
             },
-            expected_status=http_client.OK)
+            expected_status=200)
 
         self.assertValidUserResponse(r)
 
@@ -889,7 +879,7 @@ class LegacyV2UsernameTests(object):
                     'enabled': enabled,
                 },
             },
-            expected_status=http_client.OK)
+            expected_status=200)
 
         self.assertValidUserResponse(r)
 
@@ -919,7 +909,7 @@ class LegacyV2UsernameTests(object):
                     'enabled': enabled,
                 },
             },
-            expected_status=http_client.OK)
+            expected_status=200)
 
         self.assertValidUserResponse(r)
 
@@ -939,7 +929,7 @@ class LegacyV2UsernameTests(object):
                     'enabled': True,
                 },
             },
-            expected_status=http_client.OK)
+            expected_status=200)
 
         self.assertValidUserResponse(r)
 
@@ -964,7 +954,7 @@ class LegacyV2UsernameTests(object):
                     'enabled': enabled,
                 },
             },
-            expected_status=http_client.OK)
+            expected_status=200)
 
         self.assertValidUserResponse(r)
 
@@ -986,15 +976,7 @@ class RestfulTestCase(rest.RestfulTestCase):
             self.role_admin['id'])
 
 
-class V2TestCase(object):
-
-    def config_overrides(self):
-        super(V2TestCase, self).config_overrides()
-        self.config_fixture.config(
-            group='catalog',
-            driver='templated',
-            template_file=unit.dirs.tests('default_catalog.templates'))
-
+class V2TestCase(RestfulTestCase, CoreApiTests, LegacyV2UsernameTests):
     def _get_user_id(self, r):
         return r['user']['id']
 
@@ -1020,7 +1002,7 @@ class V2TestCase(object):
 
     def assertValidExtension(self, extension, expected):
         super(V2TestCase, self).assertValidExtension(extension)
-        descriptions = [ext['description'] for ext in expected.values()]
+        descriptions = [ext['description'] for ext in six.itervalues(expected)]
         description = extension.get('description')
         self.assertIsNotNone(description)
         self.assertIn(description, descriptions)
@@ -1159,9 +1141,8 @@ class V2TestCase(object):
         return r.result['user'][attribute_name]
 
     def test_service_crud_requires_auth(self):
-        """Service CRUD should return unauthorized without an X-Auth-Token."""
-        # values here don't matter because it will be unauthorized before
-        # they're checked (bug 1006822).
+        """Service CRUD should 401 without an X-Auth-Token (bug 1006822)."""
+        # values here don't matter because we should 401 before they're checked
         service_path = '/v2.0/OS-KSADM/services/%s' % uuid.uuid4().hex
         service_body = {
             'OS-KSADM:service': {
@@ -1172,43 +1153,41 @@ class V2TestCase(object):
 
         r = self.admin_request(method='GET',
                                path='/v2.0/OS-KSADM/services',
-                               expected_status=http_client.UNAUTHORIZED)
+                               expected_status=401)
         self.assertValidErrorResponse(r)
 
         r = self.admin_request(method='POST',
                                path='/v2.0/OS-KSADM/services',
                                body=service_body,
-                               expected_status=http_client.UNAUTHORIZED)
+                               expected_status=401)
         self.assertValidErrorResponse(r)
 
         r = self.admin_request(method='GET',
                                path=service_path,
-                               expected_status=http_client.UNAUTHORIZED)
+                               expected_status=401)
         self.assertValidErrorResponse(r)
 
         r = self.admin_request(method='DELETE',
                                path=service_path,
-                               expected_status=http_client.UNAUTHORIZED)
+                               expected_status=401)
         self.assertValidErrorResponse(r)
 
     def test_user_role_list_requires_auth(self):
-        """User role list return unauthorized without an X-Auth-Token."""
-        # values here don't matter because it will be unauthorized before
-        # they're checked (bug 1006815).
+        """User role list should 401 without an X-Auth-Token (bug 1006815)."""
+        # values here don't matter because we should 401 before they're checked
         path = '/v2.0/tenants/%(tenant_id)s/users/%(user_id)s/roles' % {
             'tenant_id': uuid.uuid4().hex,
             'user_id': uuid.uuid4().hex,
         }
 
-        r = self.admin_request(path=path,
-                               expected_status=http_client.UNAUTHORIZED)
+        r = self.admin_request(path=path, expected_status=401)
         self.assertValidErrorResponse(r)
 
     def test_fetch_revocation_list_nonadmin_fails(self):
         self.admin_request(
             method='GET',
             path='/v2.0/tokens/revoked',
-            expected_status=http_client.UNAUTHORIZED)
+            expected_status=401)
 
     def test_fetch_revocation_list_admin_200(self):
         token = self.get_scoped_token()
@@ -1216,12 +1195,72 @@ class V2TestCase(object):
             method='GET',
             path='/v2.0/tokens/revoked',
             token=token,
-            expected_status=http_client.OK)
+            expected_status=200)
         self.assertValidRevocationListResponse(r)
 
     def assertValidRevocationListResponse(self, response):
         self.assertIsNotNone(response.result['signed'])
 
+    def _fetch_parse_revocation_list(self):
+
+        token1 = self.get_scoped_token()
+
+        # TODO(morganfainberg): Because this is making a restful call to the
+        # app a change to UTCNOW via mock.patch will not affect the returned
+        # token. The only surefire way to ensure there is not a transient bug
+        # based upon when the second token is issued is with a sleep. This
+        # issue all stems from the limited resolution (no microseconds) on the
+        # expiry time of tokens and the way revocation events utilizes token
+        # expiry to revoke individual tokens. This is a stop-gap until all
+        # associated issues with resolution on expiration and revocation events
+        # are resolved.
+        time.sleep(1)
+
+        token2 = self.get_scoped_token()
+
+        self.admin_request(method='DELETE',
+                           path='/v2.0/tokens/%s' % token2,
+                           token=token1)
+
+        r = self.admin_request(
+            method='GET',
+            path='/v2.0/tokens/revoked',
+            token=token1,
+            expected_status=200)
+        signed_text = r.result['signed']
+
+        data_json = cms.cms_verify(signed_text, CONF.signing.certfile,
+                                   CONF.signing.ca_certs)
+
+        data = json.loads(data_json)
+
+        return (data, token2)
+
+    def test_fetch_revocation_list_md5(self):
+        """If the server is configured for md5, then the revocation list has
+           tokens hashed with MD5.
+        """
+
+        # The default hash algorithm is md5.
+        hash_algorithm = 'md5'
+
+        (data, token) = self._fetch_parse_revocation_list()
+        token_hash = cms.cms_hash_token(token, mode=hash_algorithm)
+        self.assertThat(token_hash, matchers.Equals(data['revoked'][0]['id']))
+
+    def test_fetch_revocation_list_sha256(self):
+        """If the server is configured for sha256, then the revocation list has
+           tokens hashed with SHA256
+        """
+
+        hash_algorithm = 'sha256'
+        self.config_fixture.config(group='token',
+                                   hash_algorithm=hash_algorithm)
+
+        (data, token) = self._fetch_parse_revocation_list()
+        token_hash = cms.cms_hash_token(token, mode=hash_algorithm)
+        self.assertThat(token_hash, matchers.Equals(data['revoked'][0]['id']))
+
     def test_create_update_user_invalid_enabled_type(self):
         # Enforce usage of boolean for 'enabled' field
         token = self.get_scoped_token()
@@ -1239,7 +1278,7 @@ class V2TestCase(object):
                 },
             },
             token=token,
-            expected_status=http_client.BAD_REQUEST)
+            expected_status=400)
         self.assertValidErrorResponse(r)
 
         # Test UPDATE request
@@ -1255,7 +1294,7 @@ class V2TestCase(object):
                 },
             },
             token=token,
-            expected_status=http_client.BAD_REQUEST)
+            expected_status=400)
         self.assertValidErrorResponse(r)
 
     def test_authenticating_a_user_with_an_OSKSADM_password(self):
@@ -1289,7 +1328,7 @@ class V2TestCase(object):
                     },
                 },
             },
-            expected_status=http_client.OK)
+            expected_status=200)
 
         # ensure password doesn't leak
         user_id = r.result['user']['id']
@@ -1297,7 +1336,7 @@ class V2TestCase(object):
             method='GET',
             path='/v2.0/users/%s' % user_id,
             token=token,
-            expected_status=http_client.OK)
+            expected_status=200)
         self.assertNotIn('OS-KSADM:password', r.result['user'])
 
     def test_updating_a_user_with_an_OSKSADM_password(self):
@@ -1316,7 +1355,7 @@ class V2TestCase(object):
                 },
             },
             token=token,
-            expected_status=http_client.OK)
+            expected_status=200)
 
         # successfully authenticate
         self.public_request(
@@ -1330,101 +1369,35 @@ class V2TestCase(object):
                     },
                 },
             },
-            expected_status=http_client.OK)
-
-    def test_enable_or_disable_user(self):
-        token = self.get_scoped_token()
-        user_id = self.user_badguy['id']
-
-        self.assertFalse(self.user_badguy['enabled'])
-
-        def _admin_request(body, status):
-            resp = self.admin_request(
-                method='PUT',
-                path='/v2.0/users/%s/OS-KSADM/enabled' % user_id,
-                token=token,
-                body=body,
-                expected_status=status)
-            return resp
-
-        # Enable the user.
-        body = {'user': {'enabled': True}}
-        resp = _admin_request(body, http_client.OK)
-        self.assertTrue(resp.json['user']['enabled'])
-
-        # Disable the user.
-        body = {'user': {'enabled': False}}
-        resp = _admin_request(body, http_client.OK)
-        self.assertFalse(resp.json['user']['enabled'])
-
-        # Attributes other than `enabled` should still work due to bug 1607751
-        body = {
-            'user': {
-                'description': uuid.uuid4().hex,
-                'name': uuid.uuid4().hex,
-                'enabled': True
-            }
-        }
-        _admin_request(body, http_client.OK)
-
-        #  `enabled` is boolean, type other than boolean is not allowed.
-        body = {'user': {'enabled': uuid.uuid4().hex}}
-        _admin_request(body, http_client.BAD_REQUEST)
+            expected_status=200)
 
 
-class V2TestCaseUUID(V2TestCase, RestfulTestCase, CoreApiTests,
-                     LegacyV2UsernameTests):
-
+class RevokeApiTestCase(V2TestCase):
     def config_overrides(self):
-        super(V2TestCaseUUID, self).config_overrides()
-        self.config_fixture.config(group='token', provider='uuid')
+        super(RevokeApiTestCase, self).config_overrides()
+        self.config_fixture.config(
+            group='revoke',
+            driver='keystone.contrib.revoke.backends.kvs.Revoke')
+        self.config_fixture.config(
+            group='token',
+            provider='keystone.token.providers.pki.Provider',
+            revoke_by_id=False)
 
-
-class V2TestCaseFernet(V2TestCase, RestfulTestCase, CoreApiTests,
-                       LegacyV2UsernameTests):
-
-    def config_overrides(self):
-        super(V2TestCaseFernet, self).config_overrides()
-        self.config_fixture.config(group='token', provider='fernet')
-        self.useFixture(
-            ksfixtures.KeyRepository(
-                self.config_fixture,
-                'fernet_tokens',
-                CONF.fernet_tokens.max_active_keys
-            )
-        )
+    def test_fetch_revocation_list_admin_200(self):
+        self.skipTest('Revoke API disables revocation_list.')
 
     def test_fetch_revocation_list_md5(self):
-        self.skipTest('Revocation lists do not support Fernet')
+        self.skipTest('Revoke API disables revocation_list.')
 
     def test_fetch_revocation_list_sha256(self):
-        self.skipTest('Revocation lists do not support Fernet')
+        self.skipTest('Revoke API disables revocation_list.')
 
 
 class TestFernetTokenProviderV2(RestfulTestCase):
 
     def setUp(self):
         super(TestFernetTokenProviderV2, self).setUp()
-        # Add catalog data
-        self.region = unit.new_region_ref()
-        self.region_id = self.region['id']
-        self.catalog_api.create_region(self.region)
-
-        self.service = unit.new_service_ref()
-        self.service_id = self.service['id']
-        self.catalog_api.create_service(self.service_id, self.service)
-
-        self.endpoint = unit.new_endpoint_ref(service_id=self.service_id,
-                                              interface='public',
-                                              region_id=self.region_id)
-        self.endpoint_id = self.endpoint['id']
-        self.catalog_api.create_endpoint(self.endpoint_id, self.endpoint)
-
-    def assertValidUnscopedTokenResponse(self, r):
-        v2.unscoped_validator.validate(r.json['access'])
-
-    def assertValidScopedTokenResponse(self, r):
-        v2.scoped_validator.validate(r.json['access'])
+        self.useFixture(ksfixtures.KeyRepository(self.config_fixture))
 
     # Used by RestfulTestCase
     def _get_token_id(self, r):
@@ -1439,14 +1412,9 @@ class TestFernetTokenProviderV2(RestfulTestCase):
 
     def config_overrides(self):
         super(TestFernetTokenProviderV2, self).config_overrides()
-        self.config_fixture.config(group='token', provider='fernet')
-        self.useFixture(
-            ksfixtures.KeyRepository(
-                self.config_fixture,
-                'fernet_tokens',
-                CONF.fernet_tokens.max_active_keys
-            )
-        )
+        self.config_fixture.config(
+            group='token',
+            provider='keystone.token.providers.fernet.Provider')
 
     def test_authenticate_unscoped_token(self):
         unscoped_token = self.get_unscoped_token()
@@ -1463,12 +1431,11 @@ class TestFernetTokenProviderV2(RestfulTestCase):
         admin_token = self.get_scoped_token(tenant_id=project_ref['id'])
         unscoped_token = self.get_unscoped_token()
         path = ('/v2.0/tokens/%s' % unscoped_token)
-        resp = self.admin_request(
+        self.admin_request(
             method='GET',
             path=path,
             token=admin_token,
-            expected_status=http_client.OK)
-        self.assertValidUnscopedTokenResponse(resp)
+            expected_status=200)
 
     def test_authenticate_scoped_token(self):
         project_ref = self.new_project_ref()
@@ -1494,12 +1461,11 @@ class TestFernetTokenProviderV2(RestfulTestCase):
         path = ('/v2.0/tokens/%s?belongsTo=%s' % (member_token,
                 project2_ref['id']))
         # Validate token belongs to project
-        resp = self.admin_request(
+        self.admin_request(
             method='GET',
             path=path,
             token=admin_token,
-            expected_status=http_client.OK)
-        self.assertValidScopedTokenResponse(resp)
+            expected_status=200)
 
     def test_token_authentication_and_validation(self):
         """Test token authentication for Fernet token provider.
@@ -1513,9 +1479,6 @@ class TestFernetTokenProviderV2(RestfulTestCase):
         self.assignment_api.add_role_to_user_and_project(self.user_foo['id'],
                                                          project_ref['id'],
                                                          self.role_admin['id'])
-        token_id = unscoped_token
-        if six.PY2:
-            token_id = token_id.encode('ascii')
         r = self.public_request(
             method='POST',
             path='/v2.0/tokens',
@@ -1523,21 +1486,20 @@ class TestFernetTokenProviderV2(RestfulTestCase):
                 'auth': {
                     'tenantName': project_ref['name'],
                     'token': {
-                        'id': token_id,
+                        'id': unscoped_token.encode('ascii')
                     }
                 }
             },
-            expected_status=http_client.OK)
+            expected_status=200)
 
         token_id = self._get_token_id(r)
         path = ('/v2.0/tokens/%s?belongsTo=%s' % (token_id, project_ref['id']))
         # Validate token belongs to project
-        resp = self.admin_request(
+        self.admin_request(
             method='GET',
             path=path,
-            token=self.get_admin_token(),
-            expected_status=http_client.OK)
-        self.assertValidScopedTokenResponse(resp)
+            token=CONF.admin_token,
+            expected_status=200)
 
     def test_rescoped_tokens_maintain_original_expiration(self):
         project_ref = self.new_project_ref()
@@ -1559,7 +1521,7 @@ class TestFernetTokenProviderV2(RestfulTestCase):
             },
             # NOTE(lbragstad): This test may need to be refactored if Keystone
             # decides to disallow rescoping using a scoped token.
-            expected_status=http_client.OK)
+            expected_status=200)
         original_token = resp.result['access']['token']['id']
         original_expiration = resp.result['access']['token']['expires']
 
@@ -1574,9 +1536,8 @@ class TestFernetTokenProviderV2(RestfulTestCase):
                     }
                 }
             },
-            expected_status=http_client.OK)
+            expected_status=200)
         rescoped_token = resp.result['access']['token']['id']
         rescoped_expiration = resp.result['access']['token']['expires']
         self.assertNotEqual(original_token, rescoped_token)
         self.assertEqual(original_expiration, rescoped_expiration)
-        self.assertValidScopedTokenResponse(resp)
